use crate::config::{hotreload, Context};
use crate::zabbix::api::*;
use launchy::Color;
use launchy::{self, Canvas, CanvasLayout, CanvasLayoutPoller, MsgPollingWrapper, Pad};
use std::time::Duration;
use tokio::sync::mpsc;
use tokio::time::sleep;

const MATRIX_WIDTH: i32 = 8;
const MATRIX_SIZE: i32 = MATRIX_WIDTH * MATRIX_WIDTH;
const REQUEST_WAITFOR: i32 = 5;
const ZERO_COLOR: f32 = 0.;
const ONE_COLOR: f32 = 1.;

const POLLER_ITER: u64 = 17;

fn log(num: i64) {
    return println!("test{} 🦀", num);
}

pub async fn run<'a>() {
    log(1);
    let ctx = Context::new().await;
    log(2);
    let (tx1, mut rx1) = mpsc::channel(1);
    log(3);
    let t1 = tokio::task::spawn(async move { test(&mut rx1).await });
    let t2 = tokio::task::spawn(async move { hotreload(ctx.clone(), &tx1).await });
    tokio::join!(t1, t2);
    log(4);
}

async fn test<'a>(ctxreceiver: &'a mut tokio::sync::mpsc::Receiver<Context>) {
    while let Some(val) = ctxreceiver.recv().await {
        println!("Fetching zabbix problems, {:?}", val.cfg);
        let zabbix_data = get_zabbix_problems(&val.cfg).await.unwrap();
        println!("{}", zabbix_data["result"][0]);
        println!("sleeping during {}", val.cfg.refresh.unwrap());
        sleep(Duration::from_secs(val.cfg.refresh.unwrap())).await;
    }
}

fn initpad() -> (CanvasLayout<'static>, CanvasLayoutPoller) {
    let (mut canvas, poller) = launchy::CanvasLayout::new_polling();
    match canvas.add_by_guess_rotated::<launchy::mini::Canvas>(0, 0, launchy::Rotation::None) {
        Ok(o) => o,
        Err(err) => {
            eprintln!(
                "Failed to connect to device, check USB connection {err}",
                err = err
            );
        }
    };
    println!("Connected to device using USB");
    clear(&mut canvas);
    (canvas, poller)
}

fn input(canvas: &mut CanvasLayout, poller: &mut CanvasLayoutPoller) {
    for color in (0u64..).map(|f| Color::red_green_color(f as f32 / 60.0 / 2.5)) {
        for msg in poller
            .iter_for_millis(POLLER_ITER)
            .filter(|msg| msg.is_press())
        {
            canvas[msg.pad()] = color;
            println!("{msg:?}", msg = msg.pad())
        }
        let _res = canvas.flush();
    }
}

/*
async fn draw(canvas: &mut CanvasLayout<'_>, ctx: &mut Context) {
    println!("Refresh rate is {} seconds", ctx.cfg.refresh.unwrap());
    loop {
        let zabbix_data_result = get_zabbix_problems(&ctx.cfg);
        let zabbix_data = match zabbix_data_result {
            Ok(zabbix_data) => zabbix_data,
            Err(_) => {
                println!(
                    "Error requesting zabbix service, retrying in {}",
                    REQUEST_WAITFOR
                );
                sleep(Duration::from_secs(REQUEST_WAITFOR as u64));
                continue;
            }
        };
        ctx.datamatrix.compute(&zabbix_data);
        clear(canvas);
        let mut max = 0;
        for (i, j) in ctx.datamatrix.layout.iter().enumerate() {
            let p = Pad {
                x: ((i as i32) % MATRIX_WIDTH),
                y: ((i as i32) / MATRIX_WIDTH) + 1,
            };
            let c = match j.status {
                5 => Color::RED,
                4 => Color::from_hue(0.05),
                3 => Color::from_hue(0.1),
                2 => Color::from_hue(1.22),
                _ => Color::GREEN,
            };
            canvas[p] = c;
            max += 1;
            if max >= MATRIX_SIZE {
                break;
            }
            if ctx.cfg.sloweffect.unwrap() {
                sleep(Duration::from_millis(15));
            }
            canvas.flush().unwrap();
        }
        ctx.hotreload();
    }
}*/

fn effect(canvas: &mut CanvasLayout, poller: &mut CanvasLayoutPoller) {
    for color in (0u64..).map(|f| Color::red_green_color(f as f32 / 60.0 / 2.5)) {
        for msg in poller.iter_for_millis(17).filter(|msg| msg.is_press()) {
            canvas[msg.pad()] = color;
            println!("{msg:?}", msg = msg.pad())
        }
        let _res = canvas.flush();

        canvas.iter().for_each(|pad| {
            let surrounding_color = pad
                .neighbors_5()
                .iter()
                .map(|&p| canvas.get(p).unwrap_or(Color::GREEN))
                .sum::<Color>()
                / 5.0
                / 1.05;

            canvas[pad] = canvas[pad].mix(surrounding_color, 0.4);
        });
    }
}

fn clear(canvas: &mut CanvasLayout) {
    for a in 0..8 {
        for b in 0..9 {
            canvas[Pad { x: a, y: b }] = Color {
                r: 0f32,
                g: 0f32,
                b: 1f32,
            };
        }
    }
    let _res = canvas.flush();
}

/*
let (send, recv): (Sender<Context>, Receiver<Context>) = channel();
match ctx.mode.as_str() {
    /*"draw" => {
        let (mut canvas, mut _poller) = initpad();
        draw(&mut canvas, c);
    }
    "input" => {
        let (mut canvas, mut poller) = initpad();
        input(&mut canvas, &mut poller);
    }
    "effect" => {
        let (mut canvas, mut poller) = initpad();
        effect(&mut canvas, &mut poller);
    }*/    "test" => loop {
        let mut data = cm.lock().unwrap();
        let future1 = test(&mut data);
        let mut data = cm.lock().unwrap();
        let future2 = data.hotreload();
        //futures::join!(future1, future2);
    },
    _ => {
        println!("No valid option choosen for mode");
        std::process::exit(1)
    }
}*/
