use serde_json::Value;
use std::fmt::{Display, Formatter, Result};

#[derive(Debug, Clone)]
pub enum _ZabbixStatus {
    _Disaster = 5,
    _High = 4,
    _Average = 3,
    _Warning = 2,
    _Info = 1,
}

#[derive(Debug, Clone)]
pub struct DataMatrix {
    pub layout: Vec<ZabbixProblems>,
}

impl DataMatrix {
    pub fn new() -> Self {
        Self { layout: Vec::new() }
    }
    pub fn compute(&mut self, input: &Value) {
        self.layout = Vec::new();
        for j in input.as_object().unwrap()["result"].as_array().unwrap() {
            let res = j.as_object().unwrap();
            let severity = res["priority"].as_str().unwrap().parse::<i64>().unwrap();
            self.layout.push(ZabbixProblems { status: severity });
        }
    }
}

#[derive(Debug, Clone, Copy)]
pub struct ZabbixProblems {
    pub status: i64,
}

impl Display for ZabbixProblems {
    fn fmt(&self, f: &mut Formatter) -> Result {
        write!(f, "status: {status:?}", status = self.status)
    }
}
